module Benchmark.CacheValue exposing (main)

import AMR.WithDenseList as WithDenseList
import AMR.WithList as WithList
import Benchmark.Fixture as Fixture
import Browser
import Html exposing (Html)
import Html.Attributes exposing (style)
import Json.Encode


encodedWithDenseList : String
encodedWithDenseList =
    List.foldl
        (\msg model ->
            Tuple.first
                (WithDenseList.update
                    Fixture.update
                    (always Cmd.none)
                    Fixture.encodeMsg
                    msg
                    model
                )
        )
        Fixture.modelWithDenseList
        Fixture.msgs
        |> WithDenseList.encode
        |> Json.Encode.encode 0


encodedWithList : String
encodedWithList =
    List.foldl
        (\msg model ->
            Tuple.first
                (WithList.update
                    Fixture.update
                    (always Cmd.none)
                    Fixture.encodeMsg
                    msg
                    model
                )
        )
        Fixture.modelWithList
        Fixture.msgs
        |> WithList.encode Fixture.encodeMsg
        |> Json.Encode.encode 0


view : Html Never
view =
    viewContainer
        [ viewText "Messages"
        , viewText (String.fromInt (List.length Fixture.msgs) ++ " values")
        , viewData Fixture.msgs
        , viewContainer
            [ viewText "WithList"
            , viewLength encodedWithList
            , viewData encodedWithList
            ]
        , viewContainer
            [ viewText "WithDenseList"
            , viewLength encodedWithDenseList
            , viewData encodedWithDenseList
            ]
        ]


viewContainer : List (Html Never) -> Html Never
viewContainer =
    Html.div
        [ style "padding" "10px"
        ]


viewText : String -> Html Never
viewText text =
    Html.div
        [ style "padding" "10px"
        , style "font-weight" "bold"
        ]
        [ Html.text text
        ]


viewLength : String -> Html Never
viewLength text =
    Html.div
        [ style "padding" "10px"
        , style "font-weight" "bold"
        ]
        [ Html.text (String.fromInt (String.length text) ++ " characters")
        ]


viewData : a -> Html Never
viewData data =
    Html.code
        [ style "padding" "10px"
        , style "word-break" "break-word"
        ]
        [ Html.text (Debug.toString data)
        ]


main : Program () () Never
main =
    Browser.sandbox
        { init = ()
        , update = always (always ())
        , view = always view
        }
